@@ -1,42 +1,39 @@
 from django.core.mail import BadHeaderError, send_mail
-from django.contrib.sites.shortcuts import get_current_site
-from base64 import urlsafe_b64encode
-from django.contrib.auth.tokens import default_token_generator
 from django.template.loader import render_to_string
 from django.utils.html import strip_tags
 from smtplib import SMTPException
 from .app_configurations import GetFieldFromSettings
 
+from .token_manager import TokenManager
+
 
 class _VerifyEmail:
     """
-    This class does four things:
-    1. creates tokens for each user.
-    2. set each user as inactive and saves it
-    3. embed encoded token with encoded email to make verification link.
-    4. sends the email to user with that link.
+    This class does two things:
+    1. set each user as inactive and saves it
+    2. sends the email to user with that link.
     """
 
     def __init__(self):
         self.settings = GetFieldFromSettings()
+        self.token_manager = TokenManager()
 
-    def __get_hashed_token(self, user):
-        return urlsafe_b64encode(str(default_token_generator.make_token(user)).encode('utf-8')).decode('utf-8')
-
-    def __make_verification_url(self, request, inactive_user, useremail):
-        token = self.__get_hashed_token(inactive_user)
-        email_enc = urlsafe_b64encode(str(useremail).encode('utf-8')).decode('utf-8')
-        link = f"/verification/user/verify-email/{email_enc}/{token}/"
-        
-        absolute_link = request.build_absolute_uri(link)
-        
-        return absolute_link
+    # Private :
+    def __send_email(self, msg, useremail):
+        subject = self.settings.get('subject')
+        try:
+            send_mail(subject, strip_tags(msg), from_email=self.settings.get('from_alias'),
+                      recipient_list=[useremail], html_message=msg)
+            return True
+        except (BadHeaderError, SMTPException):
+            return False
 
+    # Public :
     def send_verification_link(self, request, form):
         inactive_user = form.save(commit=False)
         inactive_user.is_active = False
         inactive_user.save()
-        
+
         try:
             useremail = form.cleaned_data.get(self.settings.get('email_field_name'))
             if not useremail:
@@ -46,18 +43,14 @@ class _VerifyEmail:
                     'as email field.'
                 )
 
-            verification_url = self.__make_verification_url(request, inactive_user, useremail)
-            subject = self.settings.get('subject')
+            verification_url = self.token_manager.generate_link(request, inactive_user, useremail)
             msg = render_to_string(self.settings.get('html_message_template', raise_exception=True),
                                    {"link": verification_url})
 
-            try:
-                send_mail(subject, strip_tags(msg), from_email=self.settings.get('from_alias'),
-                          recipient_list=[useremail], html_message=msg)
+            if self.__send_email(msg, useremail):
                 return inactive_user
-            except (BadHeaderError, SMTPException):
-                inactive_user.delete()
-                return False
+            inactive_user.delete()
+            return False
 
         except Exception as error:
 
@@ -65,7 +58,31 @@ class _VerifyEmail:
             if self.settings.get('debug_settings'):
                 raise Exception(error)
 
+    def resend_verification_link(self, request, encoded_email, encoded_token):
+        """
+        This method needs the previously sent link to get encoded email and token from that.
+        """
+        assert encoded_email is not None
+        assert encoded_token is not None
+        link = self.token_manager.request_new_link(request, encoded_email, encoded_token)
+        email = self.token_manager.perform_decoding(encoded_email)
+        if link:
+            msg = render_to_string(self.settings.get('html_message_template', raise_exception=True),
+                                   {"link": link})
+            try:
+                self.__send_email(msg, email)
+                return True
+            except (BadHeaderError, SMTPException):
+                return False
+        else:
+            return link
+
 
 #  These is supposed to be called outside of this module
 def send_verification_email(request, form):
     return _VerifyEmail().send_verification_link(request, form)
+
+
+#  These is supposed to be called outside of this module
+def resend_verification_email(request, encoded_email, encoded_token):
+    return _VerifyEmail().resend_verification_link(request, encoded_email, encoded_token)
